@@ -267,11 +267,11 @@ bool CGUIListBox::OnEvent(const SEvent& event)
 					default:
 						break;
 				}
-				if (Selected >= (s32)Items.size())
-					Selected = Items.size() - 1;
-				else
 				if (Selected<0)
 					Selected = 0;
+				if (Selected >= (s32)Items.size())
+					Selected = Items.size() - 1;	// will set Selected to -1 for empty listboxes which is correct
+				
 
 				recalculateScrollPos();
 
