@@ -1,4 +1,4 @@
-//! Copyright (C) 2009-2010 Gary Conway
+//! Copyright (C) 2009-2012 Gary Conway
 //! This file is part of the "Irrlicht Engine".
 //! For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -355,9 +355,11 @@ bool CImageLoaderRGB::readOffsetTables(io::IReadFile* file, rgbStruct& rgb) cons
 	rgb.TableLen = rgb.Header.Ysize * rgb.Header.Zsize ; // calc size of tables
 
 	// return error if unable to allocate tables
-	if ( !(rgb.StartTable = new u32[rgb.TableLen]) )
+	rgb.StartTable = new u32[rgb.TableLen];
+	if (!rgb.StartTable)
 		return false;
-	if ( !(rgb.LengthTable = new u32[rgb.TableLen]) )
+	rgb.LengthTable = new u32[rgb.TableLen];
+	if (!rgb.LengthTable)
 		return false;
 
 	file->seek(512);
@@ -395,7 +397,7 @@ void CImageLoaderRGB::processFile(io::IReadFile* file, rgbStruct& rgb) const
 
 #ifdef _IRR_RGB_FILE_INVERTED_IMAGE_
 	// preserve the image as stored, eg, inverted
-	for (u32 i = 0; i < rgb.Header.Ysize; ++i)
+	for (u16 i = 0; i < rgb.Header.Ysize; ++i)
 #else
 	// invert the image to make it upright
 	for (s32 i = (s32)(rgb.Header.Ysize)-1; i>=0; --i)
@@ -412,7 +414,7 @@ void CImageLoaderRGB::processFile(io::IReadFile* file, rgbStruct& rgb) const
 			readRGBrow( rgb.tmpA, i, 3, file, rgb);
 
 		// cycle thru all values for this row
-		for (u32 j = 0; j < rgb.Header.Xsize; ++j)
+		for (u16 j = 0; j < rgb.Header.Xsize; ++j)
 		{
 			if(rgb.Header.BPC == 1)
 			{
@@ -508,7 +510,7 @@ void CImageLoaderRGB::readRGBrow(u8 *buf, int y, int z, io::IReadFile* file, rgb
 		if (rgb.Header.BPC != 1)
 		{
 			u16* tmpbuf = reinterpret_cast<u16*>(buf);
-			for (u32 i=0; i<rgb.Header.Xsize; ++i)
+			for (u16 i=0; i<rgb.Header.Xsize; ++i)
 				tmpbuf[i] = os::Byteswap::byteswap(tmpbuf[i]);
 		}
 #endif
@@ -649,3 +651,4 @@ IImageLoader* createImageLoaderRGB()
 } // end namespace irr
 
 #endif
+
