@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2010 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -135,7 +135,7 @@ void CMetaTriangleSelector::removeAllTriangleSelectors()
 
 
 //! Return the scene node associated with a given triangle.
-const ISceneNode* CMetaTriangleSelector::getSceneNodeForTriangle(u32 triangleIndex) const
+ISceneNode* CMetaTriangleSelector::getSceneNodeForTriangle(u32 triangleIndex) const
 {
 	u32 totalTriangles = 0;
 
@@ -152,6 +152,37 @@ const ISceneNode* CMetaTriangleSelector::getSceneNodeForTriangle(u32 triangleInd
 }
 
 
+/* Return the number of TriangleSelectors that are inside this one, 
+Only useful for MetaTriangleSelector others return 1
+*/
+u32 CMetaTriangleSelector::getSelectorCount() const
+{
+	return TriangleSelectors.size();
+}
+
+
+/* Returns the TriangleSelector based on index based on getSelectorCount
+Only useful for MetaTriangleSelector others return 'this'
+*/
+ITriangleSelector* CMetaTriangleSelector::getSelector(u32 index)
+{
+	if (index >= TriangleSelectors.size())
+		return 0;
+	return TriangleSelectors[index];
+}
+
+
+/* Returns the TriangleSelector based on index based on getSelectorCount
+Only useful for MetaTriangleSelector others return 'this'
+*/
+const ITriangleSelector* CMetaTriangleSelector::getSelector(u32 index) const
+{
+	if (index >= TriangleSelectors.size())
+		return 0;
+	return TriangleSelectors[index];
+}
+
+
 } // end namespace scene
 } // end namespace irr
 
