@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2010 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -8,6 +8,7 @@
 #ifdef _IRR_COMPILE_WITH_SOFTWARE_
 
 #include "CSoftwareTexture.h"
+#include "CBlit.h"
 #include "os.h"
 #include "S3DVertex.h"
 
@@ -38,6 +39,11 @@ CSoftwareDriver::CSoftwareDriver(const core::dimension2d<u32>& windowSize, bool
 		ZBuffer = video::createZBuffer(BackBuffer->getDimension());
 	}
 
+	DriverAttributes->setAttribute("MaxTextures", 1);
+	DriverAttributes->setAttribute("MaxIndices", 1<<16);
+	DriverAttributes->setAttribute("MaxTextureSize", 1024);
+	DriverAttributes->setAttribute("Version", 1);
+
 	// create triangle renderers
 
 	TriangleRenderers[ETR_FLAT] = createTriangleRendererFlat(ZBuffer);
@@ -278,7 +284,7 @@ bool CSoftwareDriver::setRenderTarget(video::ITexture* texture, bool clearBackBu
 			ZBuffer->clear();
 
 		if (clearBackBuffer)
-			((video::CImage*)RenderTargetSurface)->fill( color );
+			RenderTargetSurface->fill(color);
 	}
 
 	return true;
@@ -815,7 +821,7 @@ void CSoftwareDriver::draw2DLine(const core::position2d<s32>& start,
 				const core::position2d<s32>& end,
 				SColor color)
 {
-	RenderTargetSurface->drawLine(start, end, color );
+	drawLine(RenderTargetSurface, start, end, color );
 }
 
 
@@ -839,14 +845,14 @@ void CSoftwareDriver::draw2DRectangle(SColor color, const core::rect<s32>& pos,
 		if(!p.isValid())
 			return;
 
-		RenderTargetSurface->drawRectangle(p, color);
+		drawRectangle(RenderTargetSurface, p, color);
 	}
 	else
 	{
 		if(!pos.isValid())
 			return;
 
-		RenderTargetSurface->drawRectangle(pos, color);
+		drawRectangle(RenderTargetSurface, pos, color);
 	}
 }
 
@@ -898,7 +904,7 @@ ITexture* CSoftwareDriver::addRenderTargetTexture(const core::dimension2d<u32>&
 												  const io::path& name,
 												  const ECOLOR_FORMAT format)
 {
-	CImage* img = new CImage(video::ECF_A1R5G5B5, size);
+	IImage* img = createImage(video::ECF_A1R5G5B5, size);
 	ITexture* tex = new CSoftwareTexture(img, name, true);
 	img->drop();
 	addTexture(tex);
@@ -916,11 +922,14 @@ void CSoftwareDriver::clearZBuffer()
 
 
 //! Returns an image created from the last rendered frame.
-IImage* CSoftwareDriver::createScreenShot()
+IImage* CSoftwareDriver::createScreenShot(video::ECOLOR_FORMAT format, video::E_RENDER_TARGET target)
 {
+	if (target != video::ERT_FRAME_BUFFER)
+		return 0;
+
 	if (BackBuffer)
 	{
-		CImage* tmp = new CImage(BackBuffer->getColorFormat(), BackBuffer->getDimension());
+		IImage* tmp = createImage(BackBuffer->getColorFormat(), BackBuffer->getDimension());
 		BackBuffer->copyTo(tmp);
 		return tmp;
 	}
